=== added file 'assess_model_config_tree.py'
--- assess_model_config_tree.py	1970-01-01 00:00:00 +0000
+++ assess_model_config_tree.py	2016-08-11 10:53:16 +0000
@@ -0,0 +1,104 @@
+#!/usr/bin/env python
+"""Test Model Tree Config functionality.
+
+Tests that default values can be overwritten and then unset.
+
+Ensures that a model config overwrites a controller config and a controller
+config overwrites a default.
+
+Also ensures that when reverting/unsetting the controller config it reverts to
+the most up to date model config setting.
+
+"""
+
+from __future__ import print_function
+
+import argparse
+import logging
+import sys
+
+from deploy_stack import (
+    BootstrapManager,
+)
+from utility import (
+    JujuAssertionError,
+    add_basic_testing_arguments,
+    configure_logging,
+)
+
+
+__metaclass__ = type
+
+
+log = logging.getLogger("assess_model_config_tree")
+
+
+def assess_model_config_tree(bs_manager, upload_tools):
+    """Assess model config tree functionality."""
+    # Need to update the cloud config details here so we can set something that
+    # we can use after boostrap.
+    config = {'ftp-proxy': 'abc.com'}
+    set_clouds_yaml_config(bs_manager.client, config)
+
+    with bs_manager.booted_context(upload_tools):
+        client = bs_manager.client
+        assert_config_value(client, 'ftp-proxy', 'controller', 'abc.com')
+        client.set_env_option('ftp-proxy', 'abc-model.com')
+        assert_config_value(client, 'ftp-proxy', 'model', 'abc-model.com')
+        client.unset_env_option('ftp-proxy')
+        assert_config_value(client, 'ftp-proxy', 'controller', 'abc.com')
+
+        assert_config_value(client, 'development', 'default', False)
+        client.set_env_option('development', True)
+        assert_config_value(client, 'development', 'model', True)
+        client.unset_env_option('development')
+        assert_config_value(client, 'development', 'default', False)
+
+
+def assert_config_value(client, attribute, source, value):
+    config_values = client.get_model_config()
+    try:
+        source_value = config_values[attribute]['source']
+        attr_value = config_values[attribute]['value']
+        if attr_value != value:
+            raise JujuAssertionError(
+                'Unexpected value for {}.\nGot {} instead of {}'.format(
+                    attribute, attr_value, value))
+
+        if source_value != source:
+            raise JujuAssertionError(
+                'Unexpected source for {}.\nGot {} instead of {}'.format(
+                    attribute, source_value, source))
+    except KeyError:
+        raise ValueError('Attribute {} not found in config values.'.format(
+            attribute))
+
+
+def set_clouds_yaml_config(client, config_details):
+    """Setup cloud details so it gets written to clouds.yaml at bootstrap."""
+    cloud_name = client.env.get_cloud()
+
+    extra_conf = {
+        'type': client.env.config['type'],
+        'regions': {client.env.get_region(): {}},
+        'config': config_details}
+    client.env.clouds['clouds'][cloud_name] = extra_conf
+
+
+def parse_args(argv):
+    """Parse all arguments."""
+    parser = argparse.ArgumentParser(description="Test Model Tree Config")
+    add_basic_testing_arguments(parser)
+    return parser.parse_args(argv)
+
+
+def main(argv=None):
+    args = parse_args(argv)
+    configure_logging(args.verbose)
+    bs_manager = BootstrapManager.from_args(args)
+    assess_model_config_tree(bs_manager, args.upload_tools)
+    return 0
+
+
+if __name__ == '__main__':
+    sys.exit(main())

=== modified file 'jujupy.py'
--- jujupy.py	2016-08-08 23:03:32 +0000
+++ jujupy.py	2016-08-11 10:53:16 +0000
@@ -1202,7 +1202,8 @@
 
     def get_model_config(self):
         """Return the value of the environment's configured option."""
-        return yaml.safe_load(self.get_juju_output('get-model-config'))
+        return yaml.safe_load(
+            self.get_juju_output('get-model-config', '--format', 'yaml'))
 
     def get_env_option(self, option):
         """Return the value of the environment's configured option."""
@@ -1213,6 +1214,10 @@
         option_value = "%s=%s" % (option, value)
         return self.juju('set-model-config', (option_value,))
 
+    def unset_env_option(self, option):
+        """Unset the value of the option in the environment."""
+        return self.juju('unset-model-config', (option,))
+
     def set_testing_tools_metadata_url(self):
         url = self.get_env_option('tools-metadata-url')
         if 'testing' not in url:

=== added file 'tests/test_assess_model_config_tree.py'
--- tests/test_assess_model_config_tree.py	1970-01-01 00:00:00 +0000
+++ tests/test_assess_model_config_tree.py	2016-08-11 10:53:16 +0000
@@ -0,0 +1,111 @@
+"""Tests for assess_model_config_tree module."""
+
+import argparse
+import logging
+from mock import Mock, patch
+import StringIO
+
+import assess_model_config_tree as amct
+from tests import (
+    parse_error,
+    TestCase,
+)
+from tests.test_jujupy import fake_juju_client
+from utility import JujuAssertionError
+
+
+class TestParseArgs(TestCase):
+
+    def test_default_args(self):
+        args = amct.parse_args(
+            ["an-env", "/bin/juju", "/tmp/logs", "an-env-mod"])
+        self.assertEqual(
+            args,
+            argparse.Namespace(
+                env="an-env",
+                juju_bin='/bin/juju',
+                logs='/tmp/logs',
+                temp_env_name='an-env-mod',
+                debug=False,
+                agent_stream=None,
+                agent_url=None,
+                bootstrap_host=None,
+                keep_env=False,
+                machine=[],
+                region=None,
+                series=None,
+                upload_tools=False,
+                verbose=20))
+
+    def test_help(self):
+        fake_stdout = StringIO.StringIO()
+        with parse_error(self) as fake_stderr:
+            with patch("sys.stdout", fake_stdout):
+                amct.parse_args(["--help"])
+        self.assertEqual("", fake_stderr.getvalue())
+        self.assertIn(
+            "Test Model Tree Config", fake_stdout.getvalue())
+
+
+class TestAssertConfigValue(TestCase):
+
+    def test_raises_JujuAssertionError_when_value_doesnt_match(self):
+        client = fake_juju_client()
+        with patch.object(client, 'get_model_config') as gmc_mock:
+            gmc_mock.return_value = {
+                'testattr': {'source': 'default', 'value': False}}
+            with self.assertRaises(JujuAssertionError):
+                amct.assert_config_value(client, 'testattr', 'default', True)
+
+    def test_raises_JujuAssertionError_when_source_doesnt_match(self):
+        client = fake_juju_client()
+        with patch.object(client, 'get_model_config') as gmc_mock:
+            gmc_mock.return_value = {
+                'testattr': {'source': 'default', 'value': False}}
+            with self.assertRaises(JujuAssertionError):
+                amct.assert_config_value(client, 'testattr', 'model', False)
+
+    def test_passes_when_source_and_value_match(self):
+        client = fake_juju_client()
+        with patch.object(client, 'get_model_config') as gmc_mock:
+            gmc_mock.return_value = {
+                'testattr': {'source': 'default', 'value': False}}
+            amct.assert_config_value(client, 'testattr', 'default', False)
+
+    def test_raises_ValueError_when_attribute_not_present(self):
+        client = fake_juju_client()
+        with patch.object(client, 'get_model_config') as gmc_mock:
+            gmc_mock.return_value = {}
+            with self.assertRaises(ValueError):
+                amct.assert_config_value(client, 'testattr', 'default', False)
+
+
+class TestSetCloudsYamlConfig(TestCase):
+
+    def test_appends_valid_cloud_details(self):
+        client = fake_juju_client()
+        client.env.load_yaml()
+
+        config_details = {'test': 'abc'}
+        amct.set_clouds_yaml_config(client, config_details)
+
+        cloud_name = 'foo'
+        cloud_details = client.env.clouds['clouds'][cloud_name]
+        self.assertEqual(cloud_details['type'], 'foo')
+        self.assertEqual(cloud_details['regions'], {'bar': {}})
+        self.assertEqual(cloud_details['config'], config_details)
+
+
+class TestMain(TestCase):
+
+    def test_main(self):
+        bs_manager = Mock()
+        argv = ["an-env", "/bin/juju", "/tmp/logs", "an-env-mod", "--verbose"]
+        with patch.object(amct, "configure_logging", autospec=True) as mock_cl:
+            with patch.object(amct.BootstrapManager, 'from_args',
+                              return_value=bs_manager):
+                    with patch.object(amct, 'assess_model_config_tree',
+                                      autospec=True) as mock_assess:
+                        amct.main(argv)
+        mock_cl.assert_called_once_with(logging.DEBUG)
+        mock_assess.assert_called_once_with(bs_manager, False)

=== modified file 'tests/test_jujupy.py'
--- tests/test_jujupy.py	2016-08-08 22:38:18 +0000
+++ tests/test_jujupy.py	2016-08-11 10:53:16 +0000
@@ -2778,7 +2778,10 @@
             result = client.get_model_config()
         assert_juju_call(
             self, po_mock, client, (
-                'juju', '--show-log', 'get-model-config', '-m', 'foo:foo'))
+                'juju', '--show-log',
+                'get-model-config',
+                '-m', 'foo:foo',
+                '--format', 'yaml'))
         self.assertEqual({'bar': 'baz'}, result)
 
     def test_get_env_option(self):
@@ -2805,6 +2808,17 @@
             ('juju', '--show-log', 'set-model-config', '-m', 'foo:foo',
              'tools-metadata-url=https://example.org/juju/tools'))
 
+    def test_unset_env_option(self):
+        env = JujuData('foo')
+        client = EnvJujuClient(env, None, 'juju')
+        with patch('subprocess.check_call') as mock:
+            client.unset_env_option('tools-metadata-url')
+        environ = dict(os.environ)
+        environ['JUJU_HOME'] = client.env.juju_home
+        mock.assert_called_with(
+            ('juju', '--show-log', 'unset-model-config', '-m', 'foo:foo',
+             'tools-metadata-url'))
+
     def test_set_testing_tools_metadata_url(self):
         env = JujuData(None, {'type': 'foo'})
         client = EnvJujuClient(env, None, None)

